# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal

import httpx

from ..types import image_generate_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.image_file import ImageFile

__all__ = ["ImagesResource", "AsyncImagesResource"]


class ImagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return ImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return ImagesResourceWithStreamingResponse(self)

    def generate(
        self,
        *,
        model: Union[
            Literal[
                "black-forest-labs/FLUX.1-schnell-Free",
                "black-forest-labs/FLUX.1-schnell",
                "black-forest-labs/FLUX.1.1-pro",
            ],
            str,
        ],
        prompt: str,
        disable_safety_checker: bool | Omit = omit,
        guidance_scale: float | Omit = omit,
        height: int | Omit = omit,
        image_loras: Iterable[image_generate_params.ImageLora] | Omit = omit,
        image_url: str | Omit = omit,
        n: int | Omit = omit,
        negative_prompt: str | Omit = omit,
        output_format: Literal["jpeg", "png"] | Omit = omit,
        reference_images: SequenceNotStr[str] | Omit = omit,
        response_format: Literal["base64", "url"] | Omit = omit,
        seed: int | Omit = omit,
        steps: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImageFile:
        """
        Use an image model to generate an image for a given prompt.

        Args:
          model: The model to use for image generation.

              [See all of Together AI's image models](https://docs.together.ai/docs/serverless-models#image-models)

          prompt: A description of the desired images. Maximum length varies by model.

          disable_safety_checker: If true, disables the safety checker for image generation.

          guidance_scale: Adjusts the alignment of the generated image with the input prompt. Higher
              values (e.g., 8-10) make the output more faithful to the prompt, while lower
              values (e.g., 1-5) encourage more creative freedom.

          height: Height of the image to generate in number of pixels.

          image_loras: An array of objects that define LoRAs (Low-Rank Adaptations) to influence the
              generated image.

          image_url: URL of an image to use for image models that support it.

          n: Number of image results to generate.

          negative_prompt: The prompt or prompts not to guide the image generation.

          output_format: The format of the image response. Can be either be `jpeg` or `png`. Defaults to
              `jpeg`.

          reference_images: An array of image URLs that guide the overall appearance and style of the
              generated image. These reference images influence the visual characteristics
              consistently across the generation.

          response_format: Format of the image response. Can be either a base64 string or a URL.

          seed: Seed used for generation. Can be used to reproduce image generations.

          steps: Number of generation steps.

          width: Width of the image to generate in number of pixels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/images/generations",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "disable_safety_checker": disable_safety_checker,
                    "guidance_scale": guidance_scale,
                    "height": height,
                    "image_loras": image_loras,
                    "image_url": image_url,
                    "n": n,
                    "negative_prompt": negative_prompt,
                    "output_format": output_format,
                    "reference_images": reference_images,
                    "response_format": response_format,
                    "seed": seed,
                    "steps": steps,
                    "width": width,
                },
                image_generate_params.ImageGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImageFile,
        )


class AsyncImagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncImagesResourceWithStreamingResponse(self)

    async def generate(
        self,
        *,
        model: Union[
            Literal[
                "black-forest-labs/FLUX.1-schnell-Free",
                "black-forest-labs/FLUX.1-schnell",
                "black-forest-labs/FLUX.1.1-pro",
            ],
            str,
        ],
        prompt: str,
        disable_safety_checker: bool | Omit = omit,
        guidance_scale: float | Omit = omit,
        height: int | Omit = omit,
        image_loras: Iterable[image_generate_params.ImageLora] | Omit = omit,
        image_url: str | Omit = omit,
        n: int | Omit = omit,
        negative_prompt: str | Omit = omit,
        output_format: Literal["jpeg", "png"] | Omit = omit,
        reference_images: SequenceNotStr[str] | Omit = omit,
        response_format: Literal["base64", "url"] | Omit = omit,
        seed: int | Omit = omit,
        steps: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImageFile:
        """
        Use an image model to generate an image for a given prompt.

        Args:
          model: The model to use for image generation.

              [See all of Together AI's image models](https://docs.together.ai/docs/serverless-models#image-models)

          prompt: A description of the desired images. Maximum length varies by model.

          disable_safety_checker: If true, disables the safety checker for image generation.

          guidance_scale: Adjusts the alignment of the generated image with the input prompt. Higher
              values (e.g., 8-10) make the output more faithful to the prompt, while lower
              values (e.g., 1-5) encourage more creative freedom.

          height: Height of the image to generate in number of pixels.

          image_loras: An array of objects that define LoRAs (Low-Rank Adaptations) to influence the
              generated image.

          image_url: URL of an image to use for image models that support it.

          n: Number of image results to generate.

          negative_prompt: The prompt or prompts not to guide the image generation.

          output_format: The format of the image response. Can be either be `jpeg` or `png`. Defaults to
              `jpeg`.

          reference_images: An array of image URLs that guide the overall appearance and style of the
              generated image. These reference images influence the visual characteristics
              consistently across the generation.

          response_format: Format of the image response. Can be either a base64 string or a URL.

          seed: Seed used for generation. Can be used to reproduce image generations.

          steps: Number of generation steps.

          width: Width of the image to generate in number of pixels.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/images/generations",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt": prompt,
                    "disable_safety_checker": disable_safety_checker,
                    "guidance_scale": guidance_scale,
                    "height": height,
                    "image_loras": image_loras,
                    "image_url": image_url,
                    "n": n,
                    "negative_prompt": negative_prompt,
                    "output_format": output_format,
                    "reference_images": reference_images,
                    "response_format": response_format,
                    "seed": seed,
                    "steps": steps,
                    "width": width,
                },
                image_generate_params.ImageGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImageFile,
        )


class ImagesResourceWithRawResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

        self.generate = to_raw_response_wrapper(
            images.generate,
        )


class AsyncImagesResourceWithRawResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

        self.generate = async_to_raw_response_wrapper(
            images.generate,
        )


class ImagesResourceWithStreamingResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

        self.generate = to_streamed_response_wrapper(
            images.generate,
        )


class AsyncImagesResourceWithStreamingResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

        self.generate = async_to_streamed_response_wrapper(
            images.generate,
        )



from together import Together

client = Together()

stream = client.chat.completions.create(
    messages=[
        {
            "role": "user",
            "content": "Say this is a test",
        }
    ],
    model="mistralai/Mixtral-8x7B-Instruct-v0.1",
    stream=True,
)
for chunk in stream:
    choice = chunk.choices[0]
    if choice.delta.content:
        print(choice.delta.content, end="", flush=True)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .log_probs import LogProbs
from .chat.chat_completion_usage import ChatCompletionUsage

__all__ = ["Completion", "Choice", "Prompt"]


class Choice(BaseModel):
    finish_reason: Optional[Literal["stop", "eos", "length", "tool_calls", "function_call"]] = None

    logprobs: Optional[LogProbs] = None

    seed: Optional[int] = None

    text: Optional[str] = None


class Prompt(BaseModel):
    logprobs: Optional[LogProbs] = None

    text: Optional[str] = None


class Completion(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: Literal["text.completion"]

    usage: Optional[ChatCompletionUsage] = None

    prompt: Optional[List[Prompt]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from pprint import pformat
from typing import cast, get_args
from pathlib import Path

import httpx

from together.types import FilePurpose

from ..lib import FileTypeError, UploadManager, AsyncUploadManager, check_file
from ..types import FilePurpose
from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.file_list import FileList
from ..types.file_response import FileResponse
from ..types.file_delete_response import FileDeleteResponse

__all__ = ["FilesResource", "AsyncFilesResource"]


class FilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return FilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return FilesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileResponse:
        """
        List the metadata for a single uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/files/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileList:
        """List the metadata for all uploaded data files."""
        return self._get(
            "/files",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileList,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileDeleteResponse:
        """
        Delete a previously uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/files/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileDeleteResponse,
        )

    def upload(
        self,
        file: Path | str,
        *,
        purpose: FilePurpose | str = "fine-tune",
        check: bool = True,
    ) -> FileResponse:
        if check:
            report_dict = check_file(file)
            if not report_dict["is_check_passed"]:
                raise FileTypeError(f"Invalid file supplied, failed to upload. Report:\n{pformat(report_dict)}")

        if isinstance(file, str):
            file = Path(file)

        if purpose not in get_args(FilePurpose):
            raise ValueError(f"Invalid purpose '{purpose}'. Must be one of: {get_args(FilePurpose)}")

        purpose = cast(FilePurpose, purpose)

        upload_manager = UploadManager(self._client)
        result = upload_manager.upload("/files", file, purpose)

        return FileResponse(
            id=result.id,
            bytes=result.bytes,
            created_at=result.created_at,
            filename=result.filename,
            FileType=result.file_type,
            LineCount=result.line_count,
            object=result.object,
            Processed=result.processed,
            purpose=result.purpose,
        )

    def content(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Get the contents of a single uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/binary", **(extra_headers or {})}
        return self._get(
            f"/files/{id}/content",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncFilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncFilesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileResponse:
        """
        List the metadata for a single uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/files/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileList:
        """List the metadata for all uploaded data files."""
        return await self._get(
            "/files",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileList,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FileDeleteResponse:
        """
        Delete a previously uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/files/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FileDeleteResponse,
        )

    async def upload(
        self,
        file: Path | str,
        *,
        purpose: FilePurpose | str = "fine-tune",
        check: bool = True,
    ) -> FileResponse:
        if check:
            report_dict = check_file(file)
            if not report_dict["is_check_passed"]:
                raise FileTypeError(f"Invalid file supplied, failed to upload. Report:\n{pformat(report_dict)}")

        if isinstance(file, str):
            file = Path(file)

        if purpose not in get_args(FilePurpose):
            raise ValueError(f"Invalid purpose '{purpose}'. Must be one of: {get_args(FilePurpose)}")

        purpose = cast(FilePurpose, purpose)

        upload_manager = AsyncUploadManager(self._client)
        result = await upload_manager.upload("/files", file, purpose)

        return FileResponse(
            id=result.id,
            bytes=result.bytes,
            created_at=result.created_at,
            filename=result.filename,
            FileType=result.file_type,
            LineCount=result.line_count,
            object=result.object,
            Processed=result.processed,
            purpose=result.purpose,
        )

    async def content(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Get the contents of a single uploaded data file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/binary", **(extra_headers or {})}
        return await self._get(
            f"/files/{id}/content",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class FilesResourceWithRawResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

        self.retrieve = to_raw_response_wrapper(
            files.retrieve,
        )
        self.list = to_raw_response_wrapper(
            files.list,
        )
        self.delete = to_raw_response_wrapper(
            files.delete,
        )
        self.content = to_custom_raw_response_wrapper(
            files.content,
            BinaryAPIResponse,
        )


class AsyncFilesResourceWithRawResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

        self.retrieve = async_to_raw_response_wrapper(
            files.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            files.list,
        )
        self.delete = async_to_raw_response_wrapper(
            files.delete,
        )
        self.content = async_to_custom_raw_response_wrapper(
            files.content,
            AsyncBinaryAPIResponse,
        )


class FilesResourceWithStreamingResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

        self.retrieve = to_streamed_response_wrapper(
            files.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            files.list,
        )
        self.delete = to_streamed_response_wrapper(
            files.delete,
        )
        self.content = to_custom_streamed_response_wrapper(
            files.content,
            StreamedBinaryAPIResponse,
        )


class AsyncFilesResourceWithStreamingResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

        self.retrieve = async_to_streamed_response_wrapper(
            files.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            files.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            files.delete,
        )
        self.content = async_to_custom_streamed_response_wrapper(
            files.content,
            AsyncStreamedBinaryAPIResponse,
        )



from typing import Any, Dict, List
from textwrap import wrap

import click
from tabulate import tabulate

from together import Together
from together.lib.utils import convert_bytes, convert_unix_timestamp
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@handle_api_errors("Files")
def list(ctx: click.Context) -> None:
    """List files"""
    client: Together = ctx.obj

    response = client.files.list()

    display_list: List[Dict[str, Any]] = []
    for i in response.data or []:
        display_list.append(
            {
                "File name": "\n".join(wrap(i.filename or "", width=30)),
                "File ID": i.id,
                "Size": convert_bytes(float(str(i.bytes))),  # convert to string for mypy typing
                "Created At": convert_unix_timestamp(i.created_at or 0),
                "Line Count": i.line_count,
            }
        )
    table = tabulate(display_list, headers="keys", tablefmt="grid", showindex=True)

    click.echo(table)



import json as json_lib
from typing import Any, Dict, List

import click
from tabulate import tabulate

from together import Together
from together.lib.cli.api._utils import handle_api_errors
from together.types.beta.clusters import ClusterStorage


def print_storage(storage: List[ClusterStorage]) -> None:
    data: List[Dict[str, Any]] = []
    for volume in storage:
        data.append(
            {
                "ID": volume.volume_id,
                "Name": volume.volume_name,
                "Size": volume.size_tib,
            }
        )
    click.echo(tabulate(data, headers="keys", tablefmt="grid"))


@click.command()
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Clusters Storage")
def list(ctx: click.Context, json: bool) -> None:
    """List storage volumes"""
    client: Together = ctx.obj

    response = client.beta.clusters.storage.list()

    if json:
        click.echo(json_lib.dumps(response.model_dump(), indent=2))
    else:
        print_storage(response.volumes)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cluster import Cluster as Cluster
from .cluster_create_params import ClusterCreateParams as ClusterCreateParams
from .cluster_list_response import ClusterListResponse as ClusterListResponse
from .cluster_update_params import ClusterUpdateParams as ClusterUpdateParams
from .cluster_delete_response import ClusterDeleteResponse as ClusterDeleteResponse
from .cluster_list_regions_response import ClusterListRegionsResponse as ClusterListRegionsResponse



import json as json_lib

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.argument(
    "volume-id",
    required=True,
)
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Clusters Storage")
def delete(ctx: click.Context, volume_id: str, json: bool) -> None:
    """Delete a storage volume"""
    client: Together = ctx.obj

    if json:
        response = client.beta.clusters.storage.delete(volume_id)
        click.echo(json_lib.dumps(response.model_dump(), indent=2))
        return

    storage = client.beta.clusters.storage.retrieve(volume_id)
    ctx.obj.print_storage([storage])
    if not click.confirm(f"Clusters Storage: Are you sure you want to delete storage volume {storage.volume_name}?"):
        return

    click.echo("Clusters Storage: Deleting storage volume...")
    response = client.beta.clusters.storage.delete(volume_id)

    click.echo(f"Clusters Storage: Deleted storage volume {storage.volume_name}")



import json
import pathlib
from typing import get_args

import click

from together import Together
from together.types import FilePurpose
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument(
    "file",
    type=click.Path(exists=True, file_okay=True, resolve_path=True, readable=True, dir_okay=False),
    required=True,
)
@click.option(
    "--purpose",
    type=click.Choice(get_args(FilePurpose)),
    default="fine-tune",
    help="Purpose of file upload. Acceptable values in enum `together.types.FilePurpose`. Defaults to `fine-tunes`.",
)
@click.option(
    "--check/--no-check",
    default=True,
    help="Whether to check the file before uploading.",
)
@handle_api_errors("Files")
def upload(ctx: click.Context, file: pathlib.Path, purpose: FilePurpose, check: bool) -> None:
    """Upload file"""

    client: Together = ctx.obj

    response = client.files.upload(file=file, purpose=purpose, check=check)

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .batch_job import BatchJob

__all__ = ["BatchListResponse"]

BatchListResponse: TypeAlias = List[BatchJob]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .clusters.clusters import (
    ClustersResource,
    AsyncClustersResource,
    ClustersResourceWithRawResponse,
    AsyncClustersResourceWithRawResponse,
    ClustersResourceWithStreamingResponse,
    AsyncClustersResourceWithStreamingResponse,
)

__all__ = ["BetaResource", "AsyncBetaResource"]


class BetaResource(SyncAPIResource):
    @cached_property
    def clusters(self) -> ClustersResource:
        return ClustersResource(self._client)

    @cached_property
    def with_raw_response(self) -> BetaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return BetaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BetaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return BetaResourceWithStreamingResponse(self)


class AsyncBetaResource(AsyncAPIResource):
    @cached_property
    def clusters(self) -> AsyncClustersResource:
        return AsyncClustersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBetaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncBetaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBetaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncBetaResourceWithStreamingResponse(self)


class BetaResourceWithRawResponse:
    def __init__(self, beta: BetaResource) -> None:
        self._beta = beta

    @cached_property
    def clusters(self) -> ClustersResourceWithRawResponse:
        return ClustersResourceWithRawResponse(self._beta.clusters)


class AsyncBetaResourceWithRawResponse:
    def __init__(self, beta: AsyncBetaResource) -> None:
        self._beta = beta

    @cached_property
    def clusters(self) -> AsyncClustersResourceWithRawResponse:
        return AsyncClustersResourceWithRawResponse(self._beta.clusters)


class BetaResourceWithStreamingResponse:
    def __init__(self, beta: BetaResource) -> None:
        self._beta = beta

    @cached_property
    def clusters(self) -> ClustersResourceWithStreamingResponse:
        return ClustersResourceWithStreamingResponse(self._beta.clusters)


class AsyncBetaResourceWithStreamingResponse:
    def __init__(self, beta: AsyncBetaResource) -> None:
        self._beta = beta

    @cached_property
    def clusters(self) -> AsyncClustersResourceWithStreamingResponse:
        return AsyncClustersResourceWithStreamingResponse(self._beta.clusters)



# Manually added to minimize breaking changes from V1
from ..lib.types.fine_tuning import (
    COMPLETED_STATUSES as COMPLETED_STATUSES,
    TrainingType as TrainingType,
    FinetuneEvent as FinetuneEvent,
    TrainingMethod as TrainingMethod,
    FinetuneRequest as FinetuneRequest,
    EmptyLRScheduler as EmptyLRScheduler,
    FinetuneProgress as FinetuneProgress,
    FinetuneResponse as FinetuneResponse,
    FullTrainingType as FullTrainingType,
    LoRATrainingType as LoRATrainingType,
    CosineLRScheduler as CosineLRScheduler,
    FinetuneEventType as FinetuneEventType,
    FinetuneJobStatus as FinetuneJobStatus,
    LinearLRScheduler as LinearLRScheduler,
    TrainingMethodDPO as TrainingMethodDPO,
    TrainingMethodSFT as TrainingMethodSFT,
    FinetuneEventLevels as FinetuneEventLevels,
    FinetuneLRScheduler as FinetuneLRScheduler,
    CosineLRSchedulerArgs as CosineLRSchedulerArgs,
    LinearLRSchedulerArgs as LinearLRSchedulerArgs,
    FinetuneTrainingLimits as FinetuneTrainingLimits,
    FinetuneMultimodalParams as FinetuneMultimodalParams,
    FinetuneFullTrainingLimits as FinetuneFullTrainingLimits,
    FinetuneLoraTrainingLimits as FinetuneLoraTrainingLimits,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..tools_param import ToolsParam
from ..tool_choice_param import ToolChoiceParam
from .chat_completion_structured_message_text_param import ChatCompletionStructuredMessageTextParam
from .chat_completion_structured_message_image_url_param import ChatCompletionStructuredMessageImageURLParam
from .chat_completion_structured_message_video_url_param import ChatCompletionStructuredMessageVideoURLParam

__all__ = [
    "CompletionCreateParamsBase",
    "Message",
    "MessageChatCompletionSystemMessageParam",
    "MessageChatCompletionUserMessageParam",
    "MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodal",
    "MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudio",
    "MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudioAudioURL",
    "MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudio",
    "MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudioInputAudio",
    "MessageChatCompletionAssistantMessageParam",
    "MessageChatCompletionAssistantMessageParamFunctionCall",
    "MessageChatCompletionToolMessageParam",
    "MessageChatCompletionFunctionMessageParam",
    "FunctionCall",
    "FunctionCallName",
    "ResponseFormat",
    "ResponseFormatText",
    "ResponseFormatJsonSchema",
    "ResponseFormatJsonSchemaJsonSchema",
    "ResponseFormatJsonObject",
    "ToolChoice",
    "CompletionCreateParamsNonStreaming",
    "CompletionCreateParamsStreaming",
]


class CompletionCreateParamsBase(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """A list of messages comprising the conversation so far."""

    model: Required[
        Union[
            Literal[
                "Qwen/Qwen2.5-72B-Instruct-Turbo",
                "Qwen/Qwen2.5-7B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
                "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            ],
            str,
        ]
    ]
    """The name of the model to query.

    [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)
    """

    chat_template_kwargs: object

    compliance: Literal["hipaa"]

    context_length_exceeded_behavior: Literal["truncate", "error"]
    """
    Defined the behavior of the API when max_tokens exceed the maximum context
    length of the model. When set to 'error', API will return 400 with appropriate
    error message. When set to 'truncate', override the max_tokens with maximum
    context length of the model.
    """

    echo: bool
    """If true, the response will contain the prompt.

    Can be used with `logprobs` to return prompt logprobs.
    """

    frequency_penalty: float
    """
    A number between -2.0 and 2.0 where a positive value decreases the likelihood of
    repeating tokens that have already been mentioned.
    """

    function_call: FunctionCall

    logit_bias: Dict[str, float]
    """Adjusts the likelihood of specific tokens appearing in the generated output."""

    logprobs: int
    """
    An integer between 0 and 20 of the top k tokens to return log probabilities for
    at each generation step, instead of just the sampled token. Log probabilities
    help assess model confidence in token predictions.
    """

    max_tokens: int
    """The maximum number of tokens to generate."""

    min_p: float
    """A number between 0 and 1 that can be used as an alternative to top_p and top-k."""

    n: int
    """The number of completions to generate for each prompt."""

    presence_penalty: float
    """
    A number between -2.0 and 2.0 where a positive value increases the likelihood of
    a model talking about new topics.
    """

    reasoning_effort: Literal["low", "medium", "high"]
    """
    Controls the level of reasoning effort the model should apply when generating
    responses. Higher values may result in more thoughtful and detailed responses
    but may take longer to generate.
    """

    repetition_penalty: float
    """
    A number that controls the diversity of generated text by reducing the
    likelihood of repeated sequences. Higher values decrease repetition.
    """

    response_format: ResponseFormat
    """An object specifying the format that the model must output.

    Setting to `{ "type": "json_schema", "json_schema": {...} }` enables Structured
    Outputs which ensures the model will match your supplied JSON schema. Learn more
    in the [Structured Outputs guide](https://docs.together.ai/docs/json-mode).

    Setting to `{ "type": "json_object" }` enables the older JSON mode, which
    ensures the message the model generates is valid JSON. Using `json_schema` is
    preferred for models that support it.
    """

    safety_model: str
    """The name of the moderation model used to validate tokens.

    Choose from the available moderation models found
    [here](https://docs.together.ai/docs/inference-models#moderation-models).
    """

    seed: int
    """Seed value for reproducibility."""

    stop: SequenceNotStr[str]
    """A list of string sequences that will truncate (stop) inference text output.

    For example, "</s>" will stop generation as soon as the model generates the
    given token.
    """

    temperature: float
    """
    A decimal number from 0-1 that determines the degree of randomness in the
    response. A temperature less than 1 favors more correctness and is appropriate
    for question answering or summarization. A value closer to 1 introduces more
    randomness in the output.
    """

    tool_choice: ToolChoice
    """Controls which (if any) function is called by the model.

    By default uses `auto`, which lets the model pick between generating a message
    or calling a function.
    """

    tools: Iterable[ToolsParam]
    """A list of tools the model may call.

    Currently, only functions are supported as a tool. Use this to provide a list of
    functions the model may generate JSON inputs for.
    """

    top_k: int
    """
    An integer that's used to limit the number of choices for the next predicted
    word or token. It specifies the maximum number of tokens to consider at each
    step, based on their probability of occurrence. This technique helps to speed up
    the generation process and can improve the quality of the generated text by
    focusing on the most likely options.
    """

    top_p: float
    """
    A percentage (also called the nucleus parameter) that's used to dynamically
    adjust the number of choices for each predicted token based on the cumulative
    probabilities. It specifies a probability threshold below which all less likely
    tokens are filtered out. This technique helps maintain diversity and generate
    more fluent and natural-sounding text.
    """


class MessageChatCompletionSystemMessageParam(TypedDict, total=False):
    content: Required[str]

    role: Required[Literal["system"]]

    name: str


class MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudioAudioURL(
    TypedDict, total=False
):
    url: Required[str]
    """The URL of the audio"""


class MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudio(
    TypedDict, total=False
):
    audio_url: Required[
        MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudioAudioURL
    ]

    type: Required[Literal["audio_url"]]


class MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudioInputAudio(
    TypedDict, total=False
):
    data: Required[str]
    """The base64 encoded audio data"""

    format: Required[Literal["wav"]]
    """The format of the audio data"""


class MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudio(
    TypedDict, total=False
):
    input_audio: Required[
        MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudioInputAudio
    ]

    type: Required[Literal["input_audio"]]


MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodal: TypeAlias = Union[
    ChatCompletionStructuredMessageTextParam,
    ChatCompletionStructuredMessageImageURLParam,
    ChatCompletionStructuredMessageVideoURLParam,
    MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalAudio,
    MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodalInputAudio,
]


class MessageChatCompletionUserMessageParam(TypedDict, total=False):
    content: Required[
        Union[str, Iterable[MessageChatCompletionUserMessageParamContentChatCompletionUserMessageContentMultimodal]]
    ]
    """
    The content of the message, which can either be a simple string or a structured
    format.
    """

    role: Required[Literal["user"]]

    name: str


class MessageChatCompletionAssistantMessageParamFunctionCall(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class MessageChatCompletionAssistantMessageParam(TypedDict, total=False):
    role: Required[Literal["assistant"]]

    content: Optional[str]

    function_call: MessageChatCompletionAssistantMessageParamFunctionCall

    name: str

    tool_calls: Iterable[ToolChoiceParam]


class MessageChatCompletionToolMessageParam(TypedDict, total=False):
    content: Required[str]

    role: Required[Literal["tool"]]

    tool_call_id: Required[str]

    name: str


class MessageChatCompletionFunctionMessageParam(TypedDict, total=False):
    content: Required[str]

    name: Required[str]

    role: Required[Literal["function"]]


Message: TypeAlias = Union[
    MessageChatCompletionSystemMessageParam,
    MessageChatCompletionUserMessageParam,
    MessageChatCompletionAssistantMessageParam,
    MessageChatCompletionToolMessageParam,
    MessageChatCompletionFunctionMessageParam,
]


class FunctionCallName(TypedDict, total=False):
    name: Required[str]


FunctionCall: TypeAlias = Union[Literal["none", "auto"], FunctionCallName]


class ResponseFormatText(TypedDict, total=False):
    """Default response format. Used to generate text responses."""

    type: Required[Literal["text"]]
    """The type of response format being defined. Always `text`."""


class ResponseFormatJsonSchemaJsonSchema(TypedDict, total=False):
    """Structured Outputs configuration options, including a JSON Schema."""

    name: Required[str]
    """The name of the response format.

    Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length
    of 64.
    """

    description: str
    """
    A description of what the response format is for, used by the model to determine
    how to respond in the format.
    """

    schema: Dict[str, object]
    """
    The schema for the response format, described as a JSON Schema object. Learn how
    to build JSON schemas [here](https://json-schema.org/).
    """

    strict: Optional[bool]
    """
    Whether to enable strict schema adherence when generating the output. If set to
    true, the model will always follow the exact schema defined in the `schema`
    field. Only a subset of JSON Schema is supported when `strict` is `true`. To
    learn more, read the
    [Structured Outputs guide](https://docs.together.ai/docs/json-mode).
    """


class ResponseFormatJsonSchema(TypedDict, total=False):
    """JSON Schema response format.

    Used to generate structured JSON responses.
    Learn more about [Structured Outputs](https://docs.together.ai/docs/json-mode).
    """

    json_schema: Required[ResponseFormatJsonSchemaJsonSchema]
    """Structured Outputs configuration options, including a JSON Schema."""

    type: Required[Literal["json_schema"]]
    """The type of response format being defined. Always `json_schema`."""


class ResponseFormatJsonObject(TypedDict, total=False):
    """JSON object response format.

    An older method of generating JSON responses.
    Using `json_schema` is recommended for models that support it. Note that the
    model will not generate JSON without a system or user message instructing it
    to do so.
    """

    type: Required[Literal["json_object"]]
    """The type of response format being defined. Always `json_object`."""


ResponseFormat: TypeAlias = Union[ResponseFormatText, ResponseFormatJsonSchema, ResponseFormatJsonObject]

ToolChoice: TypeAlias = Union[str, ToolChoiceParam]


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """
    If true, stream tokens as Server-Sent Events as the model generates them instead
    of waiting for the full model response. The stream terminates with
    `data: [DONE]`. If false, return a single JSON object containing the results.
    """


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """
    If true, stream tokens as Server-Sent Events as the model generates them instead
    of waiting for the full model response. The stream terminates with
    `data: [DONE]`. If false, return a single JSON object containing the results.
    """


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



import enum

# Session constants
# TIMEOUT_SECS = 600
# MAX_SESSION_LIFETIME_SECS = 180
# MAX_CONNECTION_RETRIES = 2
# MAX_RETRIES = 5
# INITIAL_RETRY_DELAY = 0.5
# MAX_RETRY_DELAY = 8.0

# API defaults
# BASE_URL = "https://api.together.xyz/v1"

# Download defaults
DOWNLOAD_BLOCK_SIZE = 10 * 1024 * 1024  # 10 MB
DISABLE_TQDM = False
MAX_DOWNLOAD_RETRIES = 5  # Maximum retries for download failures
DOWNLOAD_INITIAL_RETRY_DELAY = 1.0  # Initial retry delay in seconds
DOWNLOAD_MAX_RETRY_DELAY = 30.0  # Maximum retry delay in seconds

# Upload defaults
MAX_CONCURRENT_PARTS = 4  # Maximum concurrent parts for multipart upload

# Multipart upload constants
MIN_PART_SIZE_MB = 5  # Minimum part size (S3 requirement)
TARGET_PART_SIZE_MB = 250  # Target part size for optimal performance
MAX_MULTIPART_PARTS = 250  # Maximum parts per upload (S3 limit)
MULTIPART_UPLOAD_TIMEOUT = 300  # Timeout in seconds for uploading each part
MULTIPART_THRESHOLD_GB = 5.0  # threshold for switching to multipart upload

# Minimum number of samples required for fine-tuning file
MIN_SAMPLES = 1

# the number of bytes in a gigabyte, used to convert bytes to GB for readable comparison
NUM_BYTES_IN_GB = 2**30

# maximum number of GB sized files we support finetuning for
MAX_FILE_SIZE_GB = 50.1

# Multimodal limits
MAX_IMAGES_PER_EXAMPLE = 10
MAX_IMAGE_BYTES = 10 * 1024 * 1024  # 10MB
# Max length = Header length + base64 factor (4/3) * image bytes
MAX_BASE64_IMAGE_LENGTH = len("data:image/jpeg;base64,") + 4 * MAX_IMAGE_BYTES // 3

# expected columns for Parquet files
PARQUET_EXPECTED_COLUMNS = ["input_ids", "attention_mask", "labels"]


class DatasetFormat(enum.Enum):
    """Dataset format enum."""

    GENERAL = "general"
    CONVERSATION = "conversation"
    INSTRUCTION = "instruction"
    PREFERENCE_OPENAI = "preference_openai"


JSONL_REQUIRED_COLUMNS_MAP = {
    DatasetFormat.GENERAL: ["text"],
    DatasetFormat.CONVERSATION: ["messages"],
    DatasetFormat.INSTRUCTION: ["prompt", "completion"],
    DatasetFormat.PREFERENCE_OPENAI: [
        "input",
        "preferred_output",
        "non_preferred_output",
    ],
}
REQUIRED_COLUMNS_MESSAGE = ["role", "content"]
POSSIBLE_ROLES_CONVERSATION = ["system", "user", "assistant"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from together import Together, AsyncTogether
from tests.utils import assert_matches_type
from together.types import (
    FinetuneResponse,
    FineTuningListResponse,
    FineTuningCancelResponse,
    FineTuningDeleteResponse,
    FineTuningListEventsResponse,
    FineTuningEstimatePriceResponse,
    FineTuningListCheckpointsResponse,
)
from together._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFineTuning:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.retrieve(
            "id",
        )
        assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fine_tuning.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.list()
        assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.delete(
            id="id",
        )
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    def test_method_delete_with_all_params(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.delete(
            id="id",
            force=True,
        )
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fine_tuning.with_raw_response.delete(
                id="",
            )

    @parametrize
    def test_method_cancel(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.cancel(
            "id",
        )
        assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.cancel(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.cancel(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fine_tuning.with_raw_response.cancel(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_content(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        fine_tuning = client.fine_tuning.content(
            ft_id="ft_id",
        )
        assert fine_tuning.is_closed
        assert fine_tuning.json() == {"foo": "bar"}
        assert cast(Any, fine_tuning.is_closed) is True
        assert isinstance(fine_tuning, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_content_with_all_params(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        fine_tuning = client.fine_tuning.content(
            ft_id="ft_id",
            checkpoint="merged",
            checkpoint_step=0,
        )
        assert fine_tuning.is_closed
        assert fine_tuning.json() == {"foo": "bar"}
        assert cast(Any, fine_tuning.is_closed) is True
        assert isinstance(fine_tuning, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_content(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        fine_tuning = client.fine_tuning.with_raw_response.content(
            ft_id="ft_id",
        )

        assert fine_tuning.is_closed is True
        assert fine_tuning.http_request.headers.get("X-Stainless-Lang") == "python"
        assert fine_tuning.json() == {"foo": "bar"}
        assert isinstance(fine_tuning, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_content(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.fine_tuning.with_streaming_response.content(
            ft_id="ft_id",
        ) as fine_tuning:
            assert not fine_tuning.is_closed
            assert fine_tuning.http_request.headers.get("X-Stainless-Lang") == "python"

            assert fine_tuning.json() == {"foo": "bar"}
            assert cast(Any, fine_tuning.is_closed) is True
            assert isinstance(fine_tuning, StreamedBinaryAPIResponse)

        assert cast(Any, fine_tuning.is_closed) is True

    @parametrize
    def test_method_estimate_price(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.estimate_price(
            training_file="training_file",
        )
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    def test_method_estimate_price_with_all_params(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.estimate_price(
            training_file="training_file",
            from_checkpoint="from_checkpoint",
            model="model",
            n_epochs=0,
            n_evals=0,
            training_method={
                "method": "sft",
                "train_on_inputs": True,
            },
            training_type={"type": "Full"},
            validation_file="validation_file",
        )
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_estimate_price(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.estimate_price(
            training_file="training_file",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_estimate_price(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.estimate_price(
            training_file="training_file",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list_checkpoints(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.list_checkpoints(
            "id",
        )
        assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_list_checkpoints(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.list_checkpoints(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_list_checkpoints(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.list_checkpoints(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_checkpoints(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fine_tuning.with_raw_response.list_checkpoints(
                "",
            )

    @parametrize
    def test_method_list_events(self, client: Together) -> None:
        fine_tuning = client.fine_tuning.list_events(
            "id",
        )
        assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

    @parametrize
    def test_raw_response_list_events(self, client: Together) -> None:
        response = client.fine_tuning.with_raw_response.list_events(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = response.parse()
        assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

    @parametrize
    def test_streaming_response_list_events(self, client: Together) -> None:
        with client.fine_tuning.with_streaming_response.list_events(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = response.parse()
            assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_events(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.fine_tuning.with_raw_response.list_events(
                "",
            )


class TestAsyncFineTuning:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.retrieve(
            "id",
        )
        assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FinetuneResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fine_tuning.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.list()
        assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningListResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.delete(
            id="id",
        )
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.delete(
            id="id",
            force=True,
        )
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningDeleteResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fine_tuning.with_raw_response.delete(
                id="",
            )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.cancel(
            "id",
        )
        assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.cancel(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.cancel(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningCancelResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fine_tuning.with_raw_response.cancel(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_content(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        fine_tuning = await async_client.fine_tuning.content(
            ft_id="ft_id",
        )
        assert fine_tuning.is_closed
        assert await fine_tuning.json() == {"foo": "bar"}
        assert cast(Any, fine_tuning.is_closed) is True
        assert isinstance(fine_tuning, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_content_with_all_params(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        fine_tuning = await async_client.fine_tuning.content(
            ft_id="ft_id",
            checkpoint="merged",
            checkpoint_step=0,
        )
        assert fine_tuning.is_closed
        assert await fine_tuning.json() == {"foo": "bar"}
        assert cast(Any, fine_tuning.is_closed) is True
        assert isinstance(fine_tuning, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_content(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        fine_tuning = await async_client.fine_tuning.with_raw_response.content(
            ft_id="ft_id",
        )

        assert fine_tuning.is_closed is True
        assert fine_tuning.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await fine_tuning.json() == {"foo": "bar"}
        assert isinstance(fine_tuning, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_content(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.get("/finetune/download").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.fine_tuning.with_streaming_response.content(
            ft_id="ft_id",
        ) as fine_tuning:
            assert not fine_tuning.is_closed
            assert fine_tuning.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await fine_tuning.json() == {"foo": "bar"}
            assert cast(Any, fine_tuning.is_closed) is True
            assert isinstance(fine_tuning, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, fine_tuning.is_closed) is True

    @parametrize
    async def test_method_estimate_price(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.estimate_price(
            training_file="training_file",
        )
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_method_estimate_price_with_all_params(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.estimate_price(
            training_file="training_file",
            from_checkpoint="from_checkpoint",
            model="model",
            n_epochs=0,
            n_evals=0,
            training_method={
                "method": "sft",
                "train_on_inputs": True,
            },
            training_type={"type": "Full"},
            validation_file="validation_file",
        )
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_estimate_price(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.estimate_price(
            training_file="training_file",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_estimate_price(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.estimate_price(
            training_file="training_file",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningEstimatePriceResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list_checkpoints(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.list_checkpoints(
            "id",
        )
        assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_list_checkpoints(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.list_checkpoints(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_list_checkpoints(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.list_checkpoints(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningListCheckpointsResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_checkpoints(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fine_tuning.with_raw_response.list_checkpoints(
                "",
            )

    @parametrize
    async def test_method_list_events(self, async_client: AsyncTogether) -> None:
        fine_tuning = await async_client.fine_tuning.list_events(
            "id",
        )
        assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_raw_response_list_events(self, async_client: AsyncTogether) -> None:
        response = await async_client.fine_tuning.with_raw_response.list_events(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fine_tuning = await response.parse()
        assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

    @parametrize
    async def test_streaming_response_list_events(self, async_client: AsyncTogether) -> None:
        async with async_client.fine_tuning.with_streaming_response.list_events(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fine_tuning = await response.parse()
            assert_matches_type(FineTuningListEventsResponse, fine_tuning, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_events(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.fine_tuning.with_raw_response.list_events(
                "",
            )



from __future__ import annotations

from typing import Any, Literal

import click
from rich import print as rprint
from click.core import ParameterSource

from together import Together
from together.types import fine_tuning_estimate_price_params as pe_params
from together.lib.utils import log_warn
from together.lib.cli.api._utils import INT_WITH_MAX, BOOL_WITH_AUTO, handle_api_errors
from together.lib.resources.fine_tuning import get_model_limits

_CONFIRMATION_MESSAGE = (
    "You are about to create a fine-tuning job. "
    "The estimated price of this job is {price}. "
    "The actual cost of your job will be determined by the model size, the number of tokens "
    "in the training file, the number of tokens in the validation file, the number of epochs, and "
    "the number of evaluations. Visit https://www.together.ai/pricing to learn more about pricing.\n"
    "{warning}"
    "You can pass `-y` or `--confirm` to your command to skip this message.\n\n"
    "Do you want to proceed?"
)

_WARNING_MESSAGE_INSUFFICIENT_FUNDS = (
    "The estimated price of this job is significantly greater than your current credit limit and balance combined. "
    "It will likely get cancelled due to insufficient funds. "
    "Consider increasing your credit limit at https://api.together.xyz/settings/profile\n"
)


@click.command()
@click.pass_context
@click.option(
    "--training-file",
    "-t",
    type=str,
    required=True,
    help="Training file ID from Files API",
)
@click.option("--model", "-m", type=str, help="Base model name")
@click.option("--n-epochs", "-ne", type=int, default=1, help="Number of epochs to train for")
@click.option("--validation-file", type=str, default="", help="Validation file ID from Files API")
@click.option("--n-evals", type=int, default=0, help="Number of evaluation loops")
@click.option("--n-checkpoints", "-c", type=int, default=1, help="Number of checkpoints to save")
@click.option("--batch-size", "-b", type=INT_WITH_MAX, default="max", help="Train batch size")
@click.option("--learning-rate", "-lr", type=float, default=1e-5, help="Learning rate")
@click.option(
    "--lr-scheduler-type",
    type=click.Choice(["linear", "cosine"]),
    default="cosine",
    help="Learning rate scheduler type",
)
@click.option(
    "--min-lr-ratio",
    type=float,
    default=0.0,
    help="The ratio of the final learning rate to the peak learning rate",
)
@click.option(
    "--scheduler-num-cycles",
    type=float,
    default=0.5,
    help="Number or fraction of cycles for the cosine learning rate scheduler.",
)
@click.option(
    "--warmup-ratio",
    type=float,
    default=0.0,
    help="Warmup ratio for the learning rate scheduler.",
)
@click.option(
    "--max-grad-norm",
    type=float,
    default=1.0,
    help="Max gradient norm to be used for gradient clipping. Set to 0 to disable.",
)
@click.option(
    "--weight-decay",
    type=float,
    default=0.0,
    help="Weight decay",
)
@click.option(
    "--lora/--no-lora",
    type=bool,
    default=True,
    help="Whether to use LoRA adapters for fine-tuning",
)
@click.option("--lora-r", type=int, default=8, help="LoRA adapters' rank")
@click.option("--lora-dropout", type=float, default=0, help="LoRA adapters' dropout")
@click.option("--lora-alpha", type=float, default=8, help="LoRA adapters' alpha")
@click.option(
    "--lora-trainable-modules",
    type=str,
    default="all-linear",
    help="Trainable modules for LoRA adapters. For example, 'all-linear', 'q_proj,v_proj'",
)
@click.option(
    "--training-method",
    type=click.Choice(["sft", "dpo"]),
    default="sft",
    help="Training method to use. Options: sft (supervised fine-tuning), dpo (Direct Preference Optimization)",
)
@click.option(
    "--dpo-beta",
    type=float,
    default=None,
    help="Beta parameter for DPO training (only used when '--training-method' is 'dpo')",
)
@click.option(
    "--dpo-normalize-logratios-by-length",
    type=bool,
    default=False,
    help=("Whether to normalize logratios by sample length (only used when '--training-method' is 'dpo')"),
)
@click.option(
    "--rpo-alpha",
    type=float,
    default=None,
    help=(
        "RPO alpha parameter of DPO training to include NLL in the loss (only used when '--training-method' is 'dpo')"
    ),
)
@click.option(
    "--simpo-gamma",
    type=float,
    default=None,
    help="SimPO gamma parameter (only used when '--training-method' is 'dpo')",
)
@click.option(
    "--suffix",
    "-s",
    type=str,
    default=None,
    help="Suffix for the fine-tuned model name",
)
@click.option("--wandb-api-key", type=str, default=None, help="Wandb API key")
@click.option("--wandb-base-url", type=str, default=None, help="Wandb base URL")
@click.option("--wandb-project-name", type=str, default=None, help="Wandb project name")
@click.option("--wandb-name", type=str, default=None, help="Wandb run name")
@click.option(
    "--confirm",
    "-y",
    type=bool,
    is_flag=True,
    default=False,
    help="Whether to skip the launch confirmation message",
)
@click.option(
    "--train-on-inputs",
    type=BOOL_WITH_AUTO,
    default=None,
    help="Whether to mask the user messages in conversational data or prompts in instruction data. "
    "`auto` will automatically determine whether to mask the inputs based on the data format.",
)
@click.option(
    "--train-vision",
    type=bool,
    default=False,
    help="Whether to train the vision encoder. Only supported for multimodal models.",
)
@click.option(
    "--from-checkpoint",
    type=str,
    default=None,
    help="The checkpoint identifier to continue training from a previous fine-tuning job. "
    "The format: {$JOB_ID/$OUTPUT_MODEL_NAME}:{$STEP}. "
    "The step value is optional, without it the final checkpoint will be used.",
)
@click.option(
    "--from-hf-model",
    type=str,
    help="The Hugging Face Hub repo to start training from. "
    "Should be as close as possible to the base model (specified by the `model` argument) "
    "in terms of architecture and size",
)
@click.option(
    "--hf-model-revision",
    type=str,
    help="The revision of the Hugging Face Hub model to continue training from. "
    "Example: hf_model_revision=None (defaults to the latest revision in `main`) "
    "or hf_model_revision='607a30d783dfa663caf39e06633721c8d4cfcd7e' (specific commit).",
)
@click.option(
    "--hf-api-token",
    type=str,
    default=None,
    help="HF API token to use for uploading a checkpoint to a private repo",
)
@click.option(
    "--hf-output-repo-name",
    type=str,
    default=None,
    help="HF repo to upload the fine-tuned model to",
)
@handle_api_errors("Fine-tuning")
def create(
    ctx: click.Context,
    training_file: str,
    validation_file: str,
    model: str | None,
    n_epochs: int,
    n_evals: int,
    n_checkpoints: int,
    batch_size: int | Literal["max"],
    learning_rate: float,
    lr_scheduler_type: Literal["linear", "cosine"],
    min_lr_ratio: float,
    scheduler_num_cycles: float,
    warmup_ratio: float,
    max_grad_norm: float,
    weight_decay: float,
    lora: bool | None,
    lora_r: int | None,
    lora_dropout: float | None,
    lora_alpha: float | None,
    lora_trainable_modules: str | None,
    train_vision: bool,
    suffix: str | None,
    wandb_api_key: str | None,
    wandb_base_url: str | None,
    wandb_project_name: str | None,
    wandb_name: str | None,
    confirm: bool | None,
    train_on_inputs: bool | Literal["auto"] | None,
    training_method: str | None,
    dpo_beta: float | None,
    dpo_normalize_logratios_by_length: bool | None,
    rpo_alpha: float | None,
    simpo_gamma: float | None,
    from_checkpoint: str | None,
    from_hf_model: str | None,
    hf_model_revision: str | None,
    hf_api_token: str | None,
    hf_output_repo_name: str | None,
) -> None:
    """Start fine-tuning"""
    client: Together = ctx.obj

    training_args: dict[str, Any] = dict(
        training_file=training_file,
        model=model,
        n_epochs=n_epochs,
        validation_file=validation_file,
        n_evals=n_evals,
        n_checkpoints=n_checkpoints,
        batch_size=batch_size,
        learning_rate=learning_rate,
        lr_scheduler_type=lr_scheduler_type,
        min_lr_ratio=min_lr_ratio,
        scheduler_num_cycles=scheduler_num_cycles,
        warmup_ratio=warmup_ratio,
        max_grad_norm=max_grad_norm,
        weight_decay=weight_decay,
        lora=lora,
        lora_r=lora_r,
        lora_dropout=lora_dropout,
        lora_alpha=lora_alpha,
        lora_trainable_modules=lora_trainable_modules,
        train_vision=train_vision,
        suffix=suffix,
        wandb_api_key=wandb_api_key,
        wandb_base_url=wandb_base_url,
        wandb_project_name=wandb_project_name,
        wandb_name=wandb_name,
        train_on_inputs=train_on_inputs,
        training_method=training_method,
        dpo_beta=dpo_beta,
        dpo_normalize_logratios_by_length=dpo_normalize_logratios_by_length,
        rpo_alpha=rpo_alpha,
        simpo_gamma=simpo_gamma,
        from_checkpoint=from_checkpoint,
        from_hf_model=from_hf_model,
        hf_model_revision=hf_model_revision,
        hf_api_token=hf_api_token,
        hf_output_repo_name=hf_output_repo_name,
    )

    if model is None and from_checkpoint is None:
        raise click.BadParameter("You must specify either a model or a checkpoint")

    model_name = model
    if from_checkpoint is not None:
        model_name = from_checkpoint.split(":")[0]

    model_limits = get_model_limits(client, str(model_name))

    if lora:
        if model_limits.lora_training is None:
            raise click.BadParameter(f"LoRA fine-tuning is not supported for the model `{model}`")
        default_values = {
            "lora_r": model_limits.lora_training.max_rank,
            "learning_rate": 1e-3,
        }

        for arg in default_values:
            arg_source = ctx.get_parameter_source("arg")  # type: ignore[attr-defined]
            if arg_source == ParameterSource.DEFAULT:
                training_args[arg] = default_values[str(arg_source)]

        if ctx.get_parameter_source("lora_alpha") == ParameterSource.DEFAULT:  # type: ignore[attr-defined]
            training_args["lora_alpha"] = training_args["lora_r"] * 2
    else:
        if model_limits.full_training is None:
            raise click.BadParameter(f"Full fine-tuning is not supported for the model `{model}`")

        for param in ["lora_r", "lora_dropout", "lora_alpha", "lora_trainable_modules"]:
            param_source = ctx.get_parameter_source(param)  # type: ignore[attr-defined]
            if param_source != ParameterSource.DEFAULT:
                raise click.BadParameter(
                    f"You set LoRA parameter `{param}` for a full fine-tuning job. "
                    f"Please change the job type with --lora or remove `{param}` from the arguments"
                )

    if n_evals <= 0 and validation_file:
        log_warn(
            "Warning: You have specified a validation file but the number of evaluation loops is set to 0. No evaluations will be performed."
        )
    elif n_evals > 0 and not validation_file:
        raise click.BadParameter("You have specified a number of evaluation loops but no validation file.")

    training_type_cls: pe_params.TrainingType
    if lora:
        training_type_cls = pe_params.TrainingTypeLoRaTrainingType(
            lora_alpha=int(lora_alpha or 0),
            lora_r=lora_r or 0,
            lora_dropout=lora_dropout or 0,
            lora_trainable_modules=lora_trainable_modules or "all-linear",
            type="Lora",
        )
    else:
        training_type_cls = pe_params.TrainingTypeFullTrainingType(
            type="Full",
        )

    training_method_cls: pe_params.TrainingMethod
    if training_method == "sft":
        training_method_cls = pe_params.TrainingMethodTrainingMethodSft(
            method="sft",
            train_on_inputs=train_on_inputs or "auto",
        )
    else:
        training_method_cls = pe_params.TrainingMethodTrainingMethodDpo(
            method="dpo",
            dpo_beta=dpo_beta or 0,
            dpo_normalize_logratios_by_length=dpo_normalize_logratios_by_length or False,
            dpo_reference_free=False,
            rpo_alpha=rpo_alpha or 0,
            simpo_gamma=simpo_gamma or 0,
        )

    if model_limits.supports_vision:
        # Don't show price estimation for multimodal models yet
        confirm = True

    finetune_price_estimation_result = client.fine_tuning.estimate_price(
        training_file=training_file,
        validation_file=validation_file,
        model=model or "",
        n_epochs=n_epochs,
        n_evals=n_evals,
        training_type=training_type_cls,
        training_method=training_method_cls,
    )
    price = click.style(
        f"${finetune_price_estimation_result.estimated_total_price:.2f}",
        bold=True,
    )
    if not finetune_price_estimation_result.allowed_to_proceed:
        warning = click.style(_WARNING_MESSAGE_INSUFFICIENT_FUNDS, fg="red", bold=True)
    else:
        warning = ""

    confirmation_message = _CONFIRMATION_MESSAGE.format(
        price=price,
        warning=warning,
    )

    if confirm or click.confirm(confirmation_message, default=True, show_default=True):
        response = client.fine_tuning.create(
            **training_args,
            verbose=True,
        )

        report_string = f"Successfully submitted a fine-tuning job {response.id}"
        # created_at reports UTC time, we use .astimezone() to convert to local time
        formatted_time = response.created_at.astimezone().strftime("%m/%d/%Y, %H:%M:%S")
        report_string += f" at {formatted_time}"
        rprint(report_string)
    else:
        click.echo("No confirmation received, stopping job launch")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["BatchJob"]


class BatchJob(BaseModel):
    id: Optional[str] = None

    completed_at: Optional[datetime] = None

    created_at: Optional[datetime] = None

    endpoint: Optional[str] = None

    error: Optional[str] = None

    error_file_id: Optional[str] = None

    file_size_bytes: Optional[int] = None
    """Size of input file in bytes"""

    input_file_id: Optional[str] = None

    job_deadline: Optional[datetime] = None

    x_model_id: Optional[str] = FieldInfo(alias="model_id", default=None)
    """Model used for processing requests"""

    output_file_id: Optional[str] = None

    progress: Optional[float] = None
    """Completion progress (0.0 to 100)"""

    status: Optional[Literal["VALIDATING", "IN_PROGRESS", "COMPLETED", "FAILED", "EXPIRED", "CANCELLED"]] = None
    """Current status of the batch job"""

    user_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Cluster", "ControlPlaneNode", "GPUWorkerNode", "Volume"]


class ControlPlaneNode(BaseModel):
    host_name: str

    memory_gib: float

    network: str

    node_id: str

    node_name: str

    num_cpu_cores: int

    status: str


class GPUWorkerNode(BaseModel):
    host_name: str

    memory_gib: float

    networks: List[str]

    node_id: str

    node_name: str

    num_cpu_cores: int

    num_gpus: int

    status: str


class Volume(BaseModel):
    size_tib: int

    status: str

    volume_id: str

    volume_name: str


class Cluster(BaseModel):
    cluster_id: str

    cluster_name: str

    cluster_type: Literal["KUBERNETES", "SLURM"]

    control_plane_nodes: List[ControlPlaneNode]

    driver_version: Literal["CUDA_12_5_555", "CUDA_12_6_560", "CUDA_12_6_565", "CUDA_12_8_570"]

    duration_hours: int

    gpu_type: Literal["H100_SXM", "H200_SXM", "RTX_6000_PCI", "L40_PCIE", "B200_SXM", "H100_SXM_INF"]

    gpu_worker_nodes: List[GPUWorkerNode]

    kube_config: str

    num_gpus: int

    region: str

    status: Literal[
        "WaitingForControlPlaneNodes",
        "WaitingForDataPlaneNodes",
        "WaitingForSubnet",
        "WaitingForSharedVolume",
        "InstallingDrivers",
        "RunningAcceptanceTests",
        "Paused",
        "OnDemandComputePaused",
        "Ready",
        "Degraded",
        "Deleting",
    ]
    """Current status of the GPU cluster."""

    volumes: List[Volume]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import sys
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from together.lib._google_colab import get_google_colab_secret

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import TogetherError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        beta,
        chat,
        jobs,
        audio,
        evals,
        files,
        images,
        models,
        rerank,
        videos,
        batches,
        hardware,
        endpoints,
        embeddings,
        completions,
        fine_tuning,
        code_interpreter,
    )
    from .resources.jobs import JobsResource, AsyncJobsResource
    from .resources.evals import EvalsResource, AsyncEvalsResource
    from .resources.files import FilesResource, AsyncFilesResource
    from .resources.images import ImagesResource, AsyncImagesResource
    from .resources.models import ModelsResource, AsyncModelsResource
    from .resources.rerank import RerankResource, AsyncRerankResource
    from .resources.videos import VideosResource, AsyncVideosResource
    from .resources.batches import BatchesResource, AsyncBatchesResource
    from .resources.hardware import HardwareResource, AsyncHardwareResource
    from .resources.beta.beta import BetaResource, AsyncBetaResource
    from .resources.chat.chat import ChatResource, AsyncChatResource
    from .resources.endpoints import EndpointsResource, AsyncEndpointsResource
    from .resources.embeddings import EmbeddingsResource, AsyncEmbeddingsResource
    from .resources.audio.audio import AudioResource, AsyncAudioResource
    from .resources.completions import CompletionsResource, AsyncCompletionsResource
    from .resources.fine_tuning import FineTuningResource, AsyncFineTuningResource
    from .resources.code_interpreter.code_interpreter import CodeInterpreterResource, AsyncCodeInterpreterResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Together",
    "AsyncTogether",
    "Client",
    "AsyncClient",
]


class Together(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Together client instance.

        This automatically infers the `api_key` argument from the `TOGETHER_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("TOGETHER_API_KEY")
        if api_key is None and "google.colab" in sys.modules:
            api_key = get_google_colab_secret("TOGETHER_API_KEY")
        if api_key is None:
            raise TogetherError(
                "The api_key client option must be set either by passing api_key to the client or by setting the TOGETHER_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("TOGETHER_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.together.xyz/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

    @cached_property
    def beta(self) -> BetaResource:
        from .resources.beta import BetaResource

        return BetaResource(self)

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def completions(self) -> CompletionsResource:
        from .resources.completions import CompletionsResource

        return CompletionsResource(self)

    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        from .resources.embeddings import EmbeddingsResource

        return EmbeddingsResource(self)

    @cached_property
    def files(self) -> FilesResource:
        from .resources.files import FilesResource

        return FilesResource(self)

    @cached_property
    def fine_tuning(self) -> FineTuningResource:
        from .resources.fine_tuning import FineTuningResource

        return FineTuningResource(self)

    @cached_property
    def code_interpreter(self) -> CodeInterpreterResource:
        from .resources.code_interpreter import CodeInterpreterResource

        return CodeInterpreterResource(self)

    @cached_property
    def images(self) -> ImagesResource:
        from .resources.images import ImagesResource

        return ImagesResource(self)

    @cached_property
    def videos(self) -> VideosResource:
        from .resources.videos import VideosResource

        return VideosResource(self)

    @cached_property
    def audio(self) -> AudioResource:
        from .resources.audio import AudioResource

        return AudioResource(self)

    @cached_property
    def models(self) -> ModelsResource:
        from .resources.models import ModelsResource

        return ModelsResource(self)

    @cached_property
    def jobs(self) -> JobsResource:
        from .resources.jobs import JobsResource

        return JobsResource(self)

    @cached_property
    def endpoints(self) -> EndpointsResource:
        from .resources.endpoints import EndpointsResource

        return EndpointsResource(self)

    @cached_property
    def hardware(self) -> HardwareResource:
        from .resources.hardware import HardwareResource

        return HardwareResource(self)

    @cached_property
    def rerank(self) -> RerankResource:
        from .resources.rerank import RerankResource

        return RerankResource(self)

    @cached_property
    def batches(self) -> BatchesResource:
        from .resources.batches import BatchesResource

        return BatchesResource(self)

    @cached_property
    def evals(self) -> EvalsResource:
        from .resources.evals import EvalsResource

        return EvalsResource(self)

    @cached_property
    def with_raw_response(self) -> TogetherWithRawResponse:
        return TogetherWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TogetherWithStreamedResponse:
        return TogetherWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncTogether(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncTogether client instance.

        This automatically infers the `api_key` argument from the `TOGETHER_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("TOGETHER_API_KEY")
        if api_key is None and "google.colab" in sys.modules:
            api_key = get_google_colab_secret("TOGETHER_API_KEY")
        if api_key is None:
            raise TogetherError(
                "The api_key client option must be set either by passing api_key to the client or by setting the TOGETHER_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("TOGETHER_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.together.xyz/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

    @cached_property
    def beta(self) -> AsyncBetaResource:
        from .resources.beta import AsyncBetaResource

        return AsyncBetaResource(self)

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def completions(self) -> AsyncCompletionsResource:
        from .resources.completions import AsyncCompletionsResource

        return AsyncCompletionsResource(self)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        from .resources.embeddings import AsyncEmbeddingsResource

        return AsyncEmbeddingsResource(self)

    @cached_property
    def files(self) -> AsyncFilesResource:
        from .resources.files import AsyncFilesResource

        return AsyncFilesResource(self)

    @cached_property
    def fine_tuning(self) -> AsyncFineTuningResource:
        from .resources.fine_tuning import AsyncFineTuningResource

        return AsyncFineTuningResource(self)

    @cached_property
    def code_interpreter(self) -> AsyncCodeInterpreterResource:
        from .resources.code_interpreter import AsyncCodeInterpreterResource

        return AsyncCodeInterpreterResource(self)

    @cached_property
    def images(self) -> AsyncImagesResource:
        from .resources.images import AsyncImagesResource

        return AsyncImagesResource(self)

    @cached_property
    def videos(self) -> AsyncVideosResource:
        from .resources.videos import AsyncVideosResource

        return AsyncVideosResource(self)

    @cached_property
    def audio(self) -> AsyncAudioResource:
        from .resources.audio import AsyncAudioResource

        return AsyncAudioResource(self)

    @cached_property
    def models(self) -> AsyncModelsResource:
        from .resources.models import AsyncModelsResource

        return AsyncModelsResource(self)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        from .resources.jobs import AsyncJobsResource

        return AsyncJobsResource(self)

    @cached_property
    def endpoints(self) -> AsyncEndpointsResource:
        from .resources.endpoints import AsyncEndpointsResource

        return AsyncEndpointsResource(self)

    @cached_property
    def hardware(self) -> AsyncHardwareResource:
        from .resources.hardware import AsyncHardwareResource

        return AsyncHardwareResource(self)

    @cached_property
    def rerank(self) -> AsyncRerankResource:
        from .resources.rerank import AsyncRerankResource

        return AsyncRerankResource(self)

    @cached_property
    def batches(self) -> AsyncBatchesResource:
        from .resources.batches import AsyncBatchesResource

        return AsyncBatchesResource(self)

    @cached_property
    def evals(self) -> AsyncEvalsResource:
        from .resources.evals import AsyncEvalsResource

        return AsyncEvalsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncTogetherWithRawResponse:
        return AsyncTogetherWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTogetherWithStreamedResponse:
        return AsyncTogetherWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class TogetherWithRawResponse:
    _client: Together

    def __init__(self, client: Together) -> None:
        self._client = client

    @cached_property
    def beta(self) -> beta.BetaResourceWithRawResponse:
        from .resources.beta import BetaResourceWithRawResponse

        return BetaResourceWithRawResponse(self._client.beta)

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithRawResponse:
        from .resources.completions import CompletionsResourceWithRawResponse

        return CompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithRawResponse:
        from .resources.embeddings import EmbeddingsResourceWithRawResponse

        return EmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def files(self) -> files.FilesResourceWithRawResponse:
        from .resources.files import FilesResourceWithRawResponse

        return FilesResourceWithRawResponse(self._client.files)

    @cached_property
    def fine_tuning(self) -> fine_tuning.FineTuningResourceWithRawResponse:
        from .resources.fine_tuning import FineTuningResourceWithRawResponse

        return FineTuningResourceWithRawResponse(self._client.fine_tuning)

    @cached_property
    def code_interpreter(self) -> code_interpreter.CodeInterpreterResourceWithRawResponse:
        from .resources.code_interpreter import CodeInterpreterResourceWithRawResponse

        return CodeInterpreterResourceWithRawResponse(self._client.code_interpreter)

    @cached_property
    def images(self) -> images.ImagesResourceWithRawResponse:
        from .resources.images import ImagesResourceWithRawResponse

        return ImagesResourceWithRawResponse(self._client.images)

    @cached_property
    def videos(self) -> videos.VideosResourceWithRawResponse:
        from .resources.videos import VideosResourceWithRawResponse

        return VideosResourceWithRawResponse(self._client.videos)

    @cached_property
    def audio(self) -> audio.AudioResourceWithRawResponse:
        from .resources.audio import AudioResourceWithRawResponse

        return AudioResourceWithRawResponse(self._client.audio)

    @cached_property
    def models(self) -> models.ModelsResourceWithRawResponse:
        from .resources.models import ModelsResourceWithRawResponse

        return ModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def jobs(self) -> jobs.JobsResourceWithRawResponse:
        from .resources.jobs import JobsResourceWithRawResponse

        return JobsResourceWithRawResponse(self._client.jobs)

    @cached_property
    def endpoints(self) -> endpoints.EndpointsResourceWithRawResponse:
        from .resources.endpoints import EndpointsResourceWithRawResponse

        return EndpointsResourceWithRawResponse(self._client.endpoints)

    @cached_property
    def hardware(self) -> hardware.HardwareResourceWithRawResponse:
        from .resources.hardware import HardwareResourceWithRawResponse

        return HardwareResourceWithRawResponse(self._client.hardware)

    @cached_property
    def rerank(self) -> rerank.RerankResourceWithRawResponse:
        from .resources.rerank import RerankResourceWithRawResponse

        return RerankResourceWithRawResponse(self._client.rerank)

    @cached_property
    def batches(self) -> batches.BatchesResourceWithRawResponse:
        from .resources.batches import BatchesResourceWithRawResponse

        return BatchesResourceWithRawResponse(self._client.batches)

    @cached_property
    def evals(self) -> evals.EvalsResourceWithRawResponse:
        from .resources.evals import EvalsResourceWithRawResponse

        return EvalsResourceWithRawResponse(self._client.evals)


class AsyncTogetherWithRawResponse:
    _client: AsyncTogether

    def __init__(self, client: AsyncTogether) -> None:
        self._client = client

    @cached_property
    def beta(self) -> beta.AsyncBetaResourceWithRawResponse:
        from .resources.beta import AsyncBetaResourceWithRawResponse

        return AsyncBetaResourceWithRawResponse(self._client.beta)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithRawResponse:
        from .resources.completions import AsyncCompletionsResourceWithRawResponse

        return AsyncCompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithRawResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithRawResponse

        return AsyncEmbeddingsResourceWithRawResponse(self._client.embeddings)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithRawResponse:
        from .resources.files import AsyncFilesResourceWithRawResponse

        return AsyncFilesResourceWithRawResponse(self._client.files)

    @cached_property
    def fine_tuning(self) -> fine_tuning.AsyncFineTuningResourceWithRawResponse:
        from .resources.fine_tuning import AsyncFineTuningResourceWithRawResponse

        return AsyncFineTuningResourceWithRawResponse(self._client.fine_tuning)

    @cached_property
    def code_interpreter(self) -> code_interpreter.AsyncCodeInterpreterResourceWithRawResponse:
        from .resources.code_interpreter import AsyncCodeInterpreterResourceWithRawResponse

        return AsyncCodeInterpreterResourceWithRawResponse(self._client.code_interpreter)

    @cached_property
    def images(self) -> images.AsyncImagesResourceWithRawResponse:
        from .resources.images import AsyncImagesResourceWithRawResponse

        return AsyncImagesResourceWithRawResponse(self._client.images)

    @cached_property
    def videos(self) -> videos.AsyncVideosResourceWithRawResponse:
        from .resources.videos import AsyncVideosResourceWithRawResponse

        return AsyncVideosResourceWithRawResponse(self._client.videos)

    @cached_property
    def audio(self) -> audio.AsyncAudioResourceWithRawResponse:
        from .resources.audio import AsyncAudioResourceWithRawResponse

        return AsyncAudioResourceWithRawResponse(self._client.audio)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithRawResponse:
        from .resources.models import AsyncModelsResourceWithRawResponse

        return AsyncModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def jobs(self) -> jobs.AsyncJobsResourceWithRawResponse:
        from .resources.jobs import AsyncJobsResourceWithRawResponse

        return AsyncJobsResourceWithRawResponse(self._client.jobs)

    @cached_property
    def endpoints(self) -> endpoints.AsyncEndpointsResourceWithRawResponse:
        from .resources.endpoints import AsyncEndpointsResourceWithRawResponse

        return AsyncEndpointsResourceWithRawResponse(self._client.endpoints)

    @cached_property
    def hardware(self) -> hardware.AsyncHardwareResourceWithRawResponse:
        from .resources.hardware import AsyncHardwareResourceWithRawResponse

        return AsyncHardwareResourceWithRawResponse(self._client.hardware)

    @cached_property
    def rerank(self) -> rerank.AsyncRerankResourceWithRawResponse:
        from .resources.rerank import AsyncRerankResourceWithRawResponse

        return AsyncRerankResourceWithRawResponse(self._client.rerank)

    @cached_property
    def batches(self) -> batches.AsyncBatchesResourceWithRawResponse:
        from .resources.batches import AsyncBatchesResourceWithRawResponse

        return AsyncBatchesResourceWithRawResponse(self._client.batches)

    @cached_property
    def evals(self) -> evals.AsyncEvalsResourceWithRawResponse:
        from .resources.evals import AsyncEvalsResourceWithRawResponse

        return AsyncEvalsResourceWithRawResponse(self._client.evals)


class TogetherWithStreamedResponse:
    _client: Together

    def __init__(self, client: Together) -> None:
        self._client = client

    @cached_property
    def beta(self) -> beta.BetaResourceWithStreamingResponse:
        from .resources.beta import BetaResourceWithStreamingResponse

        return BetaResourceWithStreamingResponse(self._client.beta)

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithStreamingResponse:
        from .resources.completions import CompletionsResourceWithStreamingResponse

        return CompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def embeddings(self) -> embeddings.EmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import EmbeddingsResourceWithStreamingResponse

        return EmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def files(self) -> files.FilesResourceWithStreamingResponse:
        from .resources.files import FilesResourceWithStreamingResponse

        return FilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def fine_tuning(self) -> fine_tuning.FineTuningResourceWithStreamingResponse:
        from .resources.fine_tuning import FineTuningResourceWithStreamingResponse

        return FineTuningResourceWithStreamingResponse(self._client.fine_tuning)

    @cached_property
    def code_interpreter(self) -> code_interpreter.CodeInterpreterResourceWithStreamingResponse:
        from .resources.code_interpreter import CodeInterpreterResourceWithStreamingResponse

        return CodeInterpreterResourceWithStreamingResponse(self._client.code_interpreter)

    @cached_property
    def images(self) -> images.ImagesResourceWithStreamingResponse:
        from .resources.images import ImagesResourceWithStreamingResponse

        return ImagesResourceWithStreamingResponse(self._client.images)

    @cached_property
    def videos(self) -> videos.VideosResourceWithStreamingResponse:
        from .resources.videos import VideosResourceWithStreamingResponse

        return VideosResourceWithStreamingResponse(self._client.videos)

    @cached_property
    def audio(self) -> audio.AudioResourceWithStreamingResponse:
        from .resources.audio import AudioResourceWithStreamingResponse

        return AudioResourceWithStreamingResponse(self._client.audio)

    @cached_property
    def models(self) -> models.ModelsResourceWithStreamingResponse:
        from .resources.models import ModelsResourceWithStreamingResponse

        return ModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def jobs(self) -> jobs.JobsResourceWithStreamingResponse:
        from .resources.jobs import JobsResourceWithStreamingResponse

        return JobsResourceWithStreamingResponse(self._client.jobs)

    @cached_property
    def endpoints(self) -> endpoints.EndpointsResourceWithStreamingResponse:
        from .resources.endpoints import EndpointsResourceWithStreamingResponse

        return EndpointsResourceWithStreamingResponse(self._client.endpoints)

    @cached_property
    def hardware(self) -> hardware.HardwareResourceWithStreamingResponse:
        from .resources.hardware import HardwareResourceWithStreamingResponse

        return HardwareResourceWithStreamingResponse(self._client.hardware)

    @cached_property
    def rerank(self) -> rerank.RerankResourceWithStreamingResponse:
        from .resources.rerank import RerankResourceWithStreamingResponse

        return RerankResourceWithStreamingResponse(self._client.rerank)

    @cached_property
    def batches(self) -> batches.BatchesResourceWithStreamingResponse:
        from .resources.batches import BatchesResourceWithStreamingResponse

        return BatchesResourceWithStreamingResponse(self._client.batches)

    @cached_property
    def evals(self) -> evals.EvalsResourceWithStreamingResponse:
        from .resources.evals import EvalsResourceWithStreamingResponse

        return EvalsResourceWithStreamingResponse(self._client.evals)


class AsyncTogetherWithStreamedResponse:
    _client: AsyncTogether

    def __init__(self, client: AsyncTogether) -> None:
        self._client = client

    @cached_property
    def beta(self) -> beta.AsyncBetaResourceWithStreamingResponse:
        from .resources.beta import AsyncBetaResourceWithStreamingResponse

        return AsyncBetaResourceWithStreamingResponse(self._client.beta)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithStreamingResponse:
        from .resources.completions import AsyncCompletionsResourceWithStreamingResponse

        return AsyncCompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def embeddings(self) -> embeddings.AsyncEmbeddingsResourceWithStreamingResponse:
        from .resources.embeddings import AsyncEmbeddingsResourceWithStreamingResponse

        return AsyncEmbeddingsResourceWithStreamingResponse(self._client.embeddings)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithStreamingResponse:
        from .resources.files import AsyncFilesResourceWithStreamingResponse

        return AsyncFilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def fine_tuning(self) -> fine_tuning.AsyncFineTuningResourceWithStreamingResponse:
        from .resources.fine_tuning import AsyncFineTuningResourceWithStreamingResponse

        return AsyncFineTuningResourceWithStreamingResponse(self._client.fine_tuning)

    @cached_property
    def code_interpreter(self) -> code_interpreter.AsyncCodeInterpreterResourceWithStreamingResponse:
        from .resources.code_interpreter import AsyncCodeInterpreterResourceWithStreamingResponse

        return AsyncCodeInterpreterResourceWithStreamingResponse(self._client.code_interpreter)

    @cached_property
    def images(self) -> images.AsyncImagesResourceWithStreamingResponse:
        from .resources.images import AsyncImagesResourceWithStreamingResponse

        return AsyncImagesResourceWithStreamingResponse(self._client.images)

    @cached_property
    def videos(self) -> videos.AsyncVideosResourceWithStreamingResponse:
        from .resources.videos import AsyncVideosResourceWithStreamingResponse

        return AsyncVideosResourceWithStreamingResponse(self._client.videos)

    @cached_property
    def audio(self) -> audio.AsyncAudioResourceWithStreamingResponse:
        from .resources.audio import AsyncAudioResourceWithStreamingResponse

        return AsyncAudioResourceWithStreamingResponse(self._client.audio)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithStreamingResponse:
        from .resources.models import AsyncModelsResourceWithStreamingResponse

        return AsyncModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def jobs(self) -> jobs.AsyncJobsResourceWithStreamingResponse:
        from .resources.jobs import AsyncJobsResourceWithStreamingResponse

        return AsyncJobsResourceWithStreamingResponse(self._client.jobs)

    @cached_property
    def endpoints(self) -> endpoints.AsyncEndpointsResourceWithStreamingResponse:
        from .resources.endpoints import AsyncEndpointsResourceWithStreamingResponse

        return AsyncEndpointsResourceWithStreamingResponse(self._client.endpoints)

    @cached_property
    def hardware(self) -> hardware.AsyncHardwareResourceWithStreamingResponse:
        from .resources.hardware import AsyncHardwareResourceWithStreamingResponse

        return AsyncHardwareResourceWithStreamingResponse(self._client.hardware)

    @cached_property
    def rerank(self) -> rerank.AsyncRerankResourceWithStreamingResponse:
        from .resources.rerank import AsyncRerankResourceWithStreamingResponse

        return AsyncRerankResourceWithStreamingResponse(self._client.rerank)

    @cached_property
    def batches(self) -> batches.AsyncBatchesResourceWithStreamingResponse:
        from .resources.batches import AsyncBatchesResourceWithStreamingResponse

        return AsyncBatchesResourceWithStreamingResponse(self._client.batches)

    @cached_property
    def evals(self) -> evals.AsyncEvalsResourceWithStreamingResponse:
        from .resources.evals import AsyncEvalsResourceWithStreamingResponse

        return AsyncEvalsResourceWithStreamingResponse(self._client.evals)


Client = Together

AsyncClient = AsyncTogether



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "EvaluationJob",
    "Results",
    "ResultsEvaluationClassifyResults",
    "ResultsEvaluationScoreResults",
    "ResultsEvaluationScoreResultsAggregatedScores",
    "ResultsEvaluationCompareResults",
    "ResultsError",
    "StatusUpdate",
]


class ResultsEvaluationClassifyResults(BaseModel):
    generation_fail_count: Optional[float] = None
    """Number of failed generations."""

    invalid_label_count: Optional[float] = None
    """Number of invalid labels"""

    judge_fail_count: Optional[float] = None
    """Number of failed judge generations"""

    label_counts: Optional[str] = None
    """JSON string representing label counts"""

    pass_percentage: Optional[float] = None
    """Pecentage of pass labels."""

    result_file_id: Optional[str] = None
    """Data File ID"""


class ResultsEvaluationScoreResultsAggregatedScores(BaseModel):
    mean_score: Optional[float] = None

    pass_percentage: Optional[float] = None

    std_score: Optional[float] = None


class ResultsEvaluationScoreResults(BaseModel):
    aggregated_scores: Optional[ResultsEvaluationScoreResultsAggregatedScores] = None

    failed_samples: Optional[float] = None
    """number of failed samples generated from model"""

    generation_fail_count: Optional[float] = None
    """Number of failed generations."""

    invalid_score_count: Optional[float] = None
    """number of invalid scores generated from model"""

    judge_fail_count: Optional[float] = None
    """Number of failed judge generations"""

    result_file_id: Optional[str] = None
    """Data File ID"""


class ResultsEvaluationCompareResults(BaseModel):
    a_wins: Optional[int] = FieldInfo(alias="A_wins", default=None)
    """Number of times model A won"""

    b_wins: Optional[int] = FieldInfo(alias="B_wins", default=None)
    """Number of times model B won"""

    generation_fail_count: Optional[float] = None
    """Number of failed generations."""

    judge_fail_count: Optional[float] = None
    """Number of failed judge generations"""

    num_samples: Optional[int] = None
    """Total number of samples compared"""

    result_file_id: Optional[str] = None
    """Data File ID"""

    ties: Optional[int] = FieldInfo(alias="Ties", default=None)
    """Number of ties"""


class ResultsError(BaseModel):
    error: Optional[str] = None


Results: TypeAlias = Union[
    ResultsEvaluationClassifyResults, ResultsEvaluationScoreResults, ResultsEvaluationCompareResults, ResultsError, None
]


class StatusUpdate(BaseModel):
    message: Optional[str] = None
    """Additional message for this update"""

    status: Optional[str] = None
    """The status at this update"""

    timestamp: Optional[datetime] = None
    """When this update occurred"""


class EvaluationJob(BaseModel):
    created_at: Optional[datetime] = None
    """When the job was created"""

    owner_id: Optional[str] = None
    """ID of the job owner (admin only)"""

    parameters: Optional[Dict[str, object]] = None
    """The parameters used for this evaluation"""

    results: Optional[Results] = None
    """Results of the evaluation (when completed)"""

    status: Optional[Literal["pending", "queued", "running", "completed", "error", "user_error"]] = None
    """Current status of the job"""

    status_updates: Optional[List[StatusUpdate]] = None
    """History of status updates (admin only)"""

    type: Optional[Literal["classify", "score", "compare"]] = None
    """The type of evaluation"""

    updated_at: Optional[datetime] = None
    """When the job was last updated"""

    workflow_id: Optional[str] = None
    """The evaluation job ID"""
